import unittest
import argparse
import contextlib
import os
from io import StringIO
from bsl2sq.cliparser import CliParser

ABS_PATH_TEMPLATE_SONAR_FILE = os.path.abspath("tests/test_conf")
ABS_PATH_FIXTURE_SONAR_FILE = os.path.join(ABS_PATH_TEMPLATE_SONAR_FILE, "template-sonar-project.properties")


class TestCliParser(unittest.TestCase):

    def setUp(self):
        self.cliparser = CliParser()
        self.count_argument = 8

    def test_create_parser(self):
        parser = self.cliparser.create_parser()
        self.assertIsInstance(parser, argparse.ArgumentParser)
        self.assertEqual(len(parser._actions), self.count_argument)

    def test_check_args_stdout(self):

        args = argparse.Namespace()

        args.sourcedirectory = ABS_PATH_TEMPLATE_SONAR_FILE
        args.parseprefix = "рн_"
        args.file = ""
        args.absolute = False
        args.unicode = True
        args.verbose = True

        temp_stdout = StringIO()

        with contextlib.redirect_stdout(temp_stdout):
            self.cliparser.check_args(args)

        output = temp_stdout.getvalue().strip()
        self.assertEqual(output, "")

    def test_check_args_file(self):

        args = argparse.Namespace()

        args.sourcedirectory = ABS_PATH_TEMPLATE_SONAR_FILE
        args.parseprefix = "рн_"
        args.file = ABS_PATH_FIXTURE_SONAR_FILE
        args.absolute = True
        args.unicode = True
        args.verbose = True

        temp_stdout = StringIO()

        with contextlib.redirect_stdout(temp_stdout):
            self.cliparser.check_args(args)

        output = temp_stdout.getvalue().strip()
        self.assertEqual(output, "")


if __name__ == "__main__":
    unittest.main()



import argparse
import sys
import os
from .__version__ import __version__
import logging


class CliParser:

    def __init__(self) -> None:
        super().__init__()
        self.__version = __version__
        self.parser = self.create_parser()

    def create_parser(self) -> argparse.ArgumentParser:

        # Создание парсера аргументов командной строки
        parser = argparse.ArgumentParser()
        parser.add_argument('sourcedirectory', type=str,
                            help='путь к корневой папке с выгруженной конфигурацией 1с')
        parser.add_argument('parseprefix', type=str,
                            help='префикс подсистем, в которых будет осуществляться поиск путей до \
                                файлов объектов метаданных')
        parser.add_argument('-f', '--file', type=str, default="",
                            help='полный путь к файлу sonar-project.properties, в который будет выполняться \
                                выгрузка путей объектов метаданных на место переменной $inclusions_line')
        parser.add_argument('-a', '--absolute', action='store_const', const=True, default=False,
                            help='в случае указания флага будут выгружаться полные пути к файлам. без флага \
                                только относительные пути')
        parser.add_argument('-u', '--unicode', action='store_const', const=True, default=False,
                            help='в случае указания флага будут выгружаться все кириллические символы в \
                                символах unicode')
        parser.add_argument('-v', '--verbose', action='store_const', const=True, default=False,
                            help='выводить описание выполняемых операций')
        parser.add_argument('-V', '--version', action='version', version=self.__version,
                            help='вывести номер версии')

        return parser

    def check_args(self, args) -> None:

        # Процедура проверки аргументов командной строки
        # Проверка существования указанной папки

        if not os.path.exists(args.sourcedirectory):
            print(args.sourcedirectory + " - папка не существует")
            quit
        # Проверка, что по указанному пути находится папка
        if not os.path.isdir(args.sourcedirectory):
            print(args.sourcedirectory + " - это не папка")
            quit
        # Проверка указания не пустого префикса для подсистем
        if len(args.parseprefix) == 0:
            print("необходимо указать не пустой префикс")
            quit
        # Проверка, что по указанному пути находится папка
        if len(args.file) != 0 and not os.path.exists(args.file):
            print("файл sonar-project.properties по указанному пути не найден")
            quit

        # Вывод подробного лога выполнения
        if args.verbose:
            logging.info(f">>> Абсолютный путь к исходным файлам проекта: {os.path.abspath(args.sourcedirectory)}")
            logging.info(f">>> Абсолютный путь к исходным файлу sonar-project.properties: {os.path.abspath(args.file)}")

    @property
    def args(self) -> argparse.ArgumentParser:
        args = self.parser.parse_args()
        self.check_args(args)
        return vars(args)



import glob
import os
import re
import xml.dom.minidom as minidom
import logging

logging.basicConfig(format=u'[%(asctime)s]  %(message)s', level=logging.INFO)


class BslFinder:

    KEYWORD_SONAR_INCLUSION = "sonar.inclusions="
    KEYWORD_INCL_LINE_END = "$inclusions_end"
    KEYWORD_INCL_LINE = "$inclusions_line"

    def __init__(self, args) -> None:
        super().__init__()
        self.args = args
        self.root_subsystems_path = os.path.join(args["sourcedirectory"], "Subsystems")

    def get_subsystems_files_paths(self) -> list:
        """ Получение списка .xml файлов подсистем конфигурации по префиксам
        """

        os.chdir(self.root_subsystems_path)
        subsystems_files_paths = []

        # Перебор всех переданных префиксов подсистем
        for prfx in self.args["parseprefix"].split(' '):
            # поиск относительного пути файлов подсистем с заданным префиксом
            subsystems_files_paths.extend(glob.glob(os.path.join("**", prfx + "*.xml"), recursive=True))

        if self.args["verbose"]:
            logging.info(f">>> Найдено подсистем для анализа: {len(subsystems_files_paths)}")

        return subsystems_files_paths

    def get_objects_names_from_subsystem(self, file) -> set:
        """ Парсинг .xml файла подсистемы конфигурации,
            для получения списка объектов в этой подсистеме
        """

        doc = minidom.parse(file)

        subsys_tag = doc.getElementsByTagName("MetaDataObject")[0].getElementsByTagName("Subsystem")[0]
        content_tag = subsys_tag.getElementsByTagName("Properties")[0].getElementsByTagName("Content")[0]

        sub_items = content_tag.getElementsByTagName("xr:Item")

        # Маска поиска guid удаленных объектов, которые остались в подсистемах, чтобы их исключить
        mask = "[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}"
        set_metadata_names = set()

        for item in sub_items:
            obj = item.childNodes[0].data
            # Проверка результата, что он является путем к объекту, а не guid
            if not re.match(mask, obj):
                set_metadata_names.add(item.childNodes[0].data)

        return set_metadata_names

    def get_list_metadata_name(self) -> list:
        """ Получение списка наименований объектов метаданных,
            в подсистемах конфигурации по префиксам
        """

        subsystems_files_paths = self.get_subsystems_files_paths()

        set_metadata_name = set()
        list_metadata_name = []

        for sub_path in subsystems_files_paths:

            # Вывод подробного лога выполнения
            if self.args["verbose"]:
                logging.info(sub_path)

            for metadata_name in self.get_objects_names_from_subsystem(sub_path):
                set_metadata_name.add(metadata_name)

        list_metadata_name = list(set_metadata_name)
        list_metadata_name.sort()

        # Вывод подробного лога выполнения
        if self.args["verbose"]:
            logging.info(f">>> Найдено объектов для анализа: {len(list_metadata_name)}")

        return list_metadata_name

    def get_bsl_files_paths(self) -> list:
        """ Получение списка путей bsl файлов,
            в подсистемах конфигурации, по которым проводится поиск
        """

        list_metadata_name = self.get_list_metadata_name()

        list_bsl_files_paths = []

        # Получение путей к bsl файлам для определенных метаданных
        for metadata_name in list_metadata_name:

            metadata_type_name = metadata_name[:metadata_name.find(".")] + "s"
            metadata_only_name = metadata_name[metadata_name.find(".") + 1:]
            metadata_rel_path = os.path.join(metadata_type_name, metadata_only_name)
            path_to_folder = os.path.join(self.args["sourcedirectory"], metadata_rel_path)

            if not os.path.exists(path_to_folder):
                continue

            os.chdir(path_to_folder)
            # поиск файлов подсистем с заданным префиксом
            bsl_files = glob.glob(os.path.join("**", "*.bsl"), recursive=True)
            # Базовая часть путу для bsl файла (абсолютная или относительно корня конфигурации)
            base_path = path_to_folder if self.args["absolute"] else metadata_rel_path

            # Преобразование путей файлов для добавление базовой части и двойных слешей в путях при работе в ОС Windows
            bsl_files = [os.path.join(base_path, bsl_file).replace("\\", "/") for bsl_file in bsl_files]

            list_bsl_files_paths.extend(bsl_files)

        # Вывод подробного лога выполнения
        if self.args["verbose"]:
            logging.info(f">>> Количество bsl модулей для проверки: {len(list_bsl_files_paths)}")

        return list_bsl_files_paths

    def get_bsl_files_line(self) -> str:
        """ Получение строки с bsl файлами,
            для подстановки в шаблон файла для sonarqube или АП
        """

        list_bsl_files_paths = self.get_bsl_files_paths()

        # Преобразование кодировки кириллических символов в юникод
        list_bsl_files_paths = [l.encode("unicode-escape").decode("utf-8") for l in list_bsl_files_paths]

        line_bsl_files = ""

        end_line = ", \\\n"
        end_line_last_line = "\n" + self.KEYWORD_INCL_LINE_END

        for bsl_file_path in list_bsl_files_paths:
            line_bsl_files = line_bsl_files + bsl_file_path + end_line

        last_end_line_index = len(line_bsl_files) - len(end_line)
        line_bsl_files = line_bsl_files[:last_end_line_index] + end_line_last_line

        return line_bsl_files

    def write_bsl_line_to_files(self) -> None:
        """ Запись строки с bsl файлами для проверки
            в указанные файлы с определенным форматом
        """

        line_bsl_files = self.get_bsl_files_line()

        with open(self.args["file"], 'r', encoding='utf-8') as sonar_properties_file_read:
            sonar_properties_text = sonar_properties_file_read.read()
            start_sublen = sonar_properties_text.find(self.KEYWORD_SONAR_INCLUSION)
            end_sublen = sonar_properties_text.find(self.KEYWORD_INCL_LINE_END)
            if end_sublen != -1:
                f_part_text = sonar_properties_text[:start_sublen + len(self.KEYWORD_SONAR_INCLUSION)]
                e_part_text = sonar_properties_text[end_sublen + len(self.KEYWORD_INCL_LINE_END):]
                sonar_properties_text = f_part_text + self.KEYWORD_INCL_LINE + e_part_text
        with open(self.args["file"], 'w', encoding='utf-8') as sonar_properties_file_write:
            sonar_properties_file_write.write(
                sonar_properties_text.replace(self.KEYWORD_INCL_LINE, line_bsl_files))

    def write_bsl_files_paths_to_stdout(self) -> None:
        """ Вывод списка объектов с bsl файлами для проверки
            в стандартный поток вывода
        """
        list_bsl_files_paths = self.get_bsl_files_paths()

        # Преобразование кодировки кириллических символов в юникод
        list_bsl_files_paths = [l.encode("unicode-escape").decode("utf-8") for l in list_bsl_files_paths
                                if self.args["unicode"]]

        for li in list_bsl_files_paths:
            print(li)

    def data_to_sq(self) -> None:
        """ Вывод списка объектов с bsl файлами для проверки
            в различные точки вывода
        """

        if len(self.args["file"]):
            self.write_bsl_line_to_files()
        else:
            self.write_bsl_files_paths_to_stdout()



import unittest
import os
import contextlib
import shutil
from io import StringIO
from bsl2sq.bslfinder import BslFinder


unittest.TestCase.maxDiff = None
ABS_PATH_TEST_CONF = os.path.abspath("tests/test_conf")
ABS_PATH_FIXTURE_SONAR_FILE = os.path.join(ABS_PATH_TEST_CONF, "fixture-sonar-project.properties")
ABS_PATH_FIXTURE_SONAR_STDOUT = os.path.join(ABS_PATH_TEST_CONF, "fixture_stdout")
ABS_PATH_TEMPLATE_SONAR_FILE = os.path.join(ABS_PATH_TEST_CONF, "template-sonar-project.properties")
ABS_PATH_SONAR_FILE = os.path.join(ABS_PATH_TEST_CONF, "sonar-project.properties")


def setUpModule():
    shutil.copyfile(ABS_PATH_TEMPLATE_SONAR_FILE, ABS_PATH_SONAR_FILE)


def tearDownModule():
    os.remove(ABS_PATH_SONAR_FILE)


class TestBslFinder(unittest.TestCase):

    def setUp(self):

        with open(ABS_PATH_FIXTURE_SONAR_FILE, 'r', encoding='utf-8') as fsp:
            self.fixture_sp_list = fsp.read().splitlines()
            self.fixture_sp_list = [l.replace(", \\", "") for l in self.fixture_sp_list]

        with open(ABS_PATH_FIXTURE_SONAR_STDOUT, 'r', encoding='utf-8') as fsd:
            self.fixture_sd_list = fsd.read().splitlines()

        args_stdout = {
            'sourcedirectory': ABS_PATH_TEST_CONF,
            'parseprefix': "рн_ пс_",
            'file': "",
            'absolute': False,
            'unicode': True,
            'verbose': True
        }

        args_file = {
            'sourcedirectory': ABS_PATH_TEST_CONF,
            'parseprefix': "рн_ пс_",
            'file': ABS_PATH_SONAR_FILE,
            'absolute': False,
            'unicode': True,
            'verbose': True
        }

        self.bslfinder_stdout = BslFinder(args_stdout)
        self.bslfinder_file = BslFinder(args_file)

        self.subsytem_file_path = os.path.join(ABS_PATH_TEST_CONF, "Subsystems/рн_Супер.xml")

        self.count_get_subsystems_files_paths = 7
        self.count_get_objects_names_from_subsystem = 2
        self.count_get_list_metadata_name = 24
        self.count_get_bsl_files_paths = 63
        self.count_get_bsl_files_line = 6849

    def test_get_subsystems_files_paths(self):
        gsfp = self.bslfinder_stdout.get_subsystems_files_paths()
        self.assertIsInstance(gsfp, list)
        self.assertEqual(len(gsfp), self.count_get_subsystems_files_paths)

    def test_get_objects_names_from_subsystem(self):
        gonfs = self.bslfinder_stdout.get_objects_names_from_subsystem(self.subsytem_file_path)
        self.assertIsInstance(gonfs, set)
        self.assertEqual(len(gonfs), self.count_get_objects_names_from_subsystem)

    def test_get_list_metadata_name(self):
        glmn = self.bslfinder_stdout.get_list_metadata_name()
        self.assertIsInstance(glmn, list)
        self.assertEqual(len(glmn), self.count_get_list_metadata_name)

    def test_get_bsl_files_paths(self):
        gbfp = self.bslfinder_stdout.get_bsl_files_paths()
        self.assertIsInstance(gbfp, list)
        self.assertEqual(len(gbfp), self.count_get_bsl_files_paths)

    def test_get_bsl_files_line(self):
        gbfl = self.bslfinder_file.get_bsl_files_line()
        self.assertIsInstance(gbfl, str)
        self.assertEqual(len(gbfl), self.count_get_bsl_files_line)

    def test_write_bsl_line_to_files(self):
        self.bslfinder_file.write_bsl_line_to_files()
        with open(self.bslfinder_file.args["file"], 'r', encoding='utf-8') as sonar_properties_file_read:
            sonar_properties_list = sonar_properties_file_read.read().splitlines()
            sonar_properties_list = [l.replace(", \\", "") for l in sonar_properties_list]
            self.assertEqual(sorted(sonar_properties_list), sorted(self.fixture_sp_list))

    def test_write_bsl_files_paths_to_stdout(self):

        temp_stdout = StringIO()

        with contextlib.redirect_stdout(temp_stdout):
            self.bslfinder_stdout.write_bsl_files_paths_to_stdout()

        output_list = temp_stdout.getvalue().splitlines()
        self.assertEqual(sorted(output_list), sorted(self.fixture_sd_list))


if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages
from os import path
from io import open
from bsl2sq.__version__ import __version__

here_path = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here_path, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='bsl2sq',
      python_requires='>=3.6.0',
      version=__version__,
      description='bsl file finder to sonarqube',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Aleksey Maksimkin',
      author_email='maximkin@mail.ru',
      url='https://github.com/brobots-corporation/bsl2sq',
      license='GPL-3.0',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Natural Language :: Russian',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      packages=find_packages(),
      tests_require=['coverage', 'unittest-xml-reporting'],
      test_suite='tests',
      entry_points={
          'console_scripts': [
              'bsl2sq=bsl2sq.app:bsl2sq'
          ]
      },
      project_urls={
          'Bug Reports': 'https://github.com/brobots-corporation/bsl2sq/issues',
          'Source': 'https://github.com/brobots-corporation/bsl2sq',
      }
      )



from .cliparser import CliParser
from .bslfinder import BslFinder


def bsl2sq() -> None:
    """ Поиск и вывод данных для анализа в sonarqube
    """

    cli_args = CliParser().args
    BslFinder(cli_args).data_to_sq()


if __name__ == "__main__":
    bsl2sq()
